#include "../../include/ctx.h"
#include "../../include/pool.h"
#include "../../include/package.h"
#include "../../include/database.h"
#include "../../include/util.h"
#include "../../include/error.h"

lm_pkg_t *lm_ctx_package_get(lm_ctx_t *ctx, char *name, char *version) {
  lm_pool_t *pool = ctx->pools;
  lm_pkg_t *pkg = NULL;

  while (NULL != pool) {
    if((pkg = lm_pool_package_find(pool, name, version)) != NULL)
      break;
    pool = pool->next;
  }

  if(NULL == pool && NULL == pkg){
    lm_error_set(LM_ERR_PkgNotFound);
    return NULL;
  }

  if(!lm_pool_package_get(pool, pkg))
    return NULL;

  return pkg;
}

bool lm_ctx_package_verify(lm_ctx_t *ctx, char *name, char *version) {
  lm_pool_t *pool = ctx->pools;
  lm_pkg_t *pkg = NULL;

  while (NULL != pool) {
    if((pkg = lm_pool_package_find(pool, name, version)) != NULL)
      break;
    pool = pool->next;
  }

  if(NULL == pool && NULL == pkg){
    lm_error_set(LM_ERR_PkgNotFound);
    return NULL;
  }

  return lm_package_verify(pkg);
}

bool lm_ctx_package_is_installed(lm_ctx_t *ctx, char *name, char *version) {
  lm_database_t *db = lm_ctx_database_new(ctx);

  if(NULL == db)
    return false;

  bool ret = lm_database_find(db, NULL, name);
  lm_database_free(db);

  return ret;
}

lm_pkg_t *lm_ctx_package_install(lm_ctx_t *ctx, char *name, char *version){

  return NULL;
}
