#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pool.h"

#include <stdbool.h>
#include <string.h>

lm_pkg_t *lm_pool_package_find(lm_pool_t *pool, char *name, char *version){
  if(NULL == name && NULL == version){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_pkg_t *cur = pool->pkg;
  while(NULL != cur){
    if(NULL == version && eq(cur->name, name))
      return cur;
    
    else if(NULL == name && eq(cur->version, version))
      return cur;
    
    else if(eq(cur->name, name) && eq(cur->version, version))
      return cur;

    cur = cur->next;
  }

  return NULL;
}

bool lm_pool_package_add(lm_pool_t *pool, lm_pkg_t *pkg){
  if(NULL == pool || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!lm_pool_path_is_empty(pool)){
    size_t path_size = strlen(pool->paths.dir)+strlen(pkg->name)+strlen(pkg->version);
    size_t name_size = path_size - strlen(pool->paths.dir);

    char archive_path[path_size+10], sig_path[path_size+20];
    char archive_name[name_size+10], sig_name[name_size+20];

    sprintf(archive_name, "%s_%s.mpf", pkg->name, pkg->version);
    sprintf(sig_name, "%s_%s.mpf.sig", pkg->name, pkg->version);

    join(archive_path, pool->paths.dir, archive_name);
    join(sig_path, pool->paths.dir, sig_name);

    lm_package_path_set_archive(pkg, archive_path);
    lm_package_path_set_signature(pkg, sig_path);
  }

  if(NULL == pool->pkg){
    pool->pkg = pkg;
    return true;
  }

  lm_pkg_t *cur = pool->pkg;
  while(NULL != cur){
    if(NULL == cur->next){
      cur->next = pkg;
      return true;
    }
    cur = cur->next;
  }

  return false;
}

bool lm_pool_package_get(lm_pool_t *pool, lm_pkg_t *pkg){
  if(NULL == pool || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(lm_package_path_is_empty(pkg)){
    lm_error_set(LM_ERR_PkgPathsEmpty);
    return false;
  }

  char data[strlen(pool->url.path)+strlen(pkg->name)+strlen(pkg->version)+20];
  char name[strlen(pkg->name)+strlen(pkg->version)+10];

  lm_mptp_t packet;
  bool ret = false;
  int sock = -1;
  
  snprintf(name, sizeof(name), "%s_%s", pkg->name, pkg->version);
  join(data, pool->url.path, name);

  if((sock = lm_mptp_client_connect(pool->url.host, pool->url.port)) < 0)
    return false;

  lm_mptp_init(&packet, true, MPTP_C2S_PULL, true);
  lm_mptp_set_host(&packet, pool->url.host);
  lm_mptp_set_data(&packet, data, strlen(data));

  if(!lm_mptp_client_send(sock, &packet))
    goto end;

  if(!lm_mptp_recvfile(sock, pkg->paths.archive))
    goto end;

  if(!lm_mptp_recvfile(sock, pkg->paths.signature))
    goto end;

  ret = true;
end:
  lm_mptp_close(sock);
  return ret;
}
